#!/bin/bash

# TO EXECUTE IN UBUNTU ENVIRONMENT (20.04)

# generate a package for cyphering OpenVPN files and store them in the Provencore TrustStore
 
# usage: ./generate_ipk.sh <GW_EUI> <p12> <server-IP> <passwd> <Forwarder_IP>

GW_EUI=$1
P12_FILE=$2
SERVER_IP=$3
PASSPHRASE=$4
FWD_IP=$5

PROV_P12_FILE=client-openvpn.p12
PROV_CONF_FILE=provencore-openvpn.conf
PROV_CLIENT_CONF_FILE=client-openvpn.conf


if [ "$1" == "" ] || [ "$2" == "" ] || [ "$3" == "" ] || [ "$4" == "" ] || [ "$5" == "" ];  then
  echo "parameters are missing"
  echo "usage: ./generate_ipk.sh <GW_EUI> <client.p12> <server_IP> <passphrase> <Forwarder_IP>"
  echo generate a package for cyphering OpenVPN files and store them in the Provencore TrustStore
  echo activate CPF and set the node IP to the requested IP value
  exit  
else
 if [[ $1 =~ 7276.*|7076.* ]]; then
   echo "Correct GW EUI"
 else
   echo "ERROR, wrong GW EUI. Must be 7276xxxxxxxxxxxx or 7076xxxxxxxxxxxx"
  exit
fi	 
fi

# check if already exist

line=$(grep $GW_EUI list_p12)
if [ "$line" != "" ]; then
  line2=$(grep $P12_FILE <<< $line)
  if [ "$line2" != "" ]; then
	echo "OK"
  else
        gw=$(echo $line2 | cut -d\; -f1) 
	echo "KO this p12 is already used by another GW: $gw"
	exit
  fi
else
  # GW_EUI not found, search for the GW that has this p12
  line2=$(grep $P12_FILE list_p12)
  if [ "$line2" != "" ]; then
         gw=$(echo $line2 | cut -d\; -f1) 
	 echo "ERROR, this p12 is already used by another GW: $gw"
    exit
  else
    echo "Adding GW in the list"
    echo "$GW_EUI;$P12_FILE" >> list_p12
  fi
fi

WORKDIR=`dirname $0`

OPKGB_WEB="https://git.yoctoproject.org/cgit/cgit.cgi/opkg-utils/plain/opkg-build"
OPKG_PKG_NAME="openvpn-config"
VERSION="1.0"
OPKG_PKG_DIR="${WORKDIR}/package"
DEST_DIR=

#ARCH defines wether the packet will be used on IBTS (klk_lpbs), iFemtoCell (klk_wifc)
# or all the Kerlink Wirnet(TM) products with KerOS FW >=v4.1 (klkgw)
ARCH="klkgw"
 
PATH=${PATH}:${WORKDIR}

# store GW_EUI
tmp=$(echo $GW_EUI | tr '[:upper:]' '[:lower:]' | cut -c 9-16) 
GW_EUI=$tmp

# rename files
cp $P12_FILE $PROV_P12_FILE 
#cp $CONF_FILE $PROV_CONF_FILE 
#cp $CLIENT_CONF_FILE $PROV_CLIENT_CONF_FILE 


# update configuration files 
cmd=$(echo "sed -i \"s/remote .*/remote $SERVER_IP 1194/g\" $PROV_CONF_FILE")
eval $cmd
cmd=$(echo "sed -i \"s/kerlinkkerlink/$PASSPHRASE/g\" $PROV_CONF_FILE")
eval $cmd
cmd=$(echo "sed -i \"s/remote .*/remote $SERVER_IP 1194/g\" $PROV_CLIENT_CONF_FILE")
eval $cmd


# cyphering files
./pnrcipher.py -f $PROV_P12_FILE -p $PASSPHRASE
./pnrcipher.py -f $PROV_CONF_FILE -p $PASSPHRASE

# 1 - Download opkg-build if necessary
command -v opkg-build > /dev/null 2>&1
if [ $? -ne 0 ]
then
	echo "Opkg-build is necessary, downloading it from ${OPKGB_WEB}"
	wget ${OPKGB_WEB}
	chmod 755 opkg-build
fi
 
# 2 - Generate meta-data for opkg
rm -rf ${OPKG_PKG_DIR}
mkdir -p ${OPKG_PKG_DIR}/CONTROL
cp control ${OPKG_PKG_DIR}/CONTROL/control
cmd=$(echo "sed -i \"s/PASSWORD=.*/PASSWORD=$PASSPHRASE/g\" ${OPKG_PKG_DIR}/CONTROL/control")
eval $cmd
cmd=$(echo "sed -i \"s/Package: configure-openvpn-spn/Package: configure-openvpn-spn-$GW_EUI/g\" ${OPKG_PKG_DIR}/CONTROL/control")
eval $cmd

 
# 3 - Copy or create the file(s) to be installed
cp -fr user  ${OPKG_PKG_DIR}
cp -fr  etc  ${OPKG_PKG_DIR}
cp $PROV_P12_FILE.enc ${OPKG_PKG_DIR}/etc/openvpn
cp $PROV_CONF_FILE.enc ${OPKG_PKG_DIR}/etc/openvpn
cp $PROV_CLIENT_CONF_FILE ${OPKG_PKG_DIR}/etc/openvpn
 

# In most cases, the user doesn't need this script to generate the files to be installed 
# He just needs to copy pre-existing files from another directory
# The previous "cat" command can be replaced by a "cp" command
# cp /custom/path/to/file_to_install ${DEST_DIR}/ 
 
# 4 - Generate or copy a preinst script
#cat > ${OPKG_PKG_DIR}/CONTROL/preinst << EOF 
#	echo "I am a script file executed before the installation of ${OPKG_PKG_NAME}-${VERSION}"
#EOF

# In most cases, the user doesn't need this script to generate the "preinst" file
# He just needs to copy a pre-existing file from another directory
# The previous "cat" command can be replaced by a "cp" command
# cp /custom/path/to/preinst ${OPKG_PKG_DIR}/CONTROL/
#chmod 755 ${OPKG_PKG_DIR}/CONTROL/preinst
 
# 5 - Generate or copy a preinst script
#cat > ${OPKG_PKG_DIR}/CONTROL/preinst << EOF1
#EOF1


# In most cases, the user doesn't need this script to generate the "postinst" file
# He just needs to copy a pre-existing file from another directory
# The previous "cat" command can be replaced by a "cp" command
# cp /custom/path/to/postinst ${OPKG_PKG_DIR}/CONTROL/

cp ${WORKDIR}/postinst ${OPKG_PKG_DIR}/CONTROL/postinst
chmod 755 ${OPKG_PKG_DIR}/CONTROL/postinst
cmd=$(echo "sed -i \"s/PASSWORD=.*/PASSWORD=$PASSPHRASE/g\" ${OPKG_PKG_DIR}/CONTROL/postinst")
eval $cmd
cmd=$(echo "sed -i \"s/FWD_IP/$FWD_IP/g\" ${OPKG_PKG_DIR}/CONTROL/postinst")
eval $cmd


# 6 - Generate or copy a postrm script
#cat > ${OPKG_PKG_DIR}/CONTROL/postrm << EOF2
#EOF2

# In most cases, the user doesn't need this script to generate the "postrm" file
# He just needs to copy a pre-existing file from another directory
# The previous "cat" command can be replaced by a "cp" command
# cp /custom/path/to/postrm ${OPKG_PKG_DIR}/CONTROL/

#chmod 755 ${OPKG_PKG_DIR}/CONTROL/postrm 

# 7 - Generate the package
rm -f *.ipk
opkg-build -o root -g root ${OPKG_PKG_DIR} > log
cat log
rm -f log
